@@ -20,6 +20,7 @@ import socket
 import sys
 import threading
 import time
+import unittest
 import warnings
 
 try:
@@ -28,20 +29,13 @@ try:
 except ImportError:
     HAVE_IPADDRESS = False
 
-if sys.version_info[:2] == (2, 6):
-    import unittest2 as unittest
-    from unittest2 import SkipTest
-else:
-    import unittest
-    from unittest import SkipTest
-
 from functools import wraps
+from unittest import SkipTest
 
 import pymongo
 import pymongo.errors
 
 from bson.son import SON
-from bson.py3compat import _unicode
 from pymongo import common, message
 from pymongo.common import partition_node
 from pymongo.ssl_support import HAVE_SSL, validate_cert_reqs
@@ -50,6 +44,14 @@ from test.version import Version
 if HAVE_SSL:
     import ssl
 
+try:
+    # Enable the fault handler to dump the traceback of each running thread
+    # after a segfault.
+    import faulthandler
+    faulthandler.enable()
+except ImportError:
+    pass
+
 # The host and port of a single mongod or mongos, or the seed host
 # for a replica set.
 host = os.environ.get("DB_IP", 'localhost')
@@ -157,7 +159,6 @@ class ClientContext(object):
         """Create a client and grab essential information from the server."""
         self.connection_attempts = []
         self.connected = False
-        self.ismaster = {}
         self.w = None
         self.nodes = set()
         self.replica_set_name = None
@@ -167,6 +168,7 @@ class ClientContext(object):
         self.auth_enabled = False
         self.test_commands_enabled = False
         self.is_mongos = False
+        self.mongoses = []
         self.is_rs = False
         self.has_ipv6 = False
         self.ssl = False
@@ -181,6 +183,10 @@ class ClientContext(object):
         if COMPRESSORS:
             self.default_client_options["compressors"] = COMPRESSORS
 
+    @property
+    def ismaster(self):
+        return self.client.admin.command('isMaster')
+
     def _connect(self, host, port, **kwargs):
         # Jython takes a long time to connect.
         if sys.platform.startswith('java'):
@@ -250,7 +256,11 @@ class ClientContext(object):
                 self.cmd_line = self.client.admin.command('getCmdLineOpts')
 
             self.server_status = self.client.admin.command('serverStatus')
-            self.ismaster = ismaster = self.client.admin.command('isMaster')
+            if self.storage_engine == "mmapv1":
+                # MMAPv1 does not support retryWrites=True.
+                self.default_client_options['retryWrites'] = False
+
+            ismaster = self.ismaster
             self.sessions_enabled = 'logicalSessionTimeoutMinutes' in ismaster
 
             if 'setName' in ismaster:
@@ -273,18 +283,17 @@ class ClientContext(object):
                         **self.default_client_options)
 
                 # Get the authoritative ismaster result from the primary.
-                self.ismaster = self.client.admin.command('ismaster')
+                ismaster = self.ismaster
                 nodes = [partition_node(node.lower())
-                         for node in self.ismaster.get('hosts', [])]
+                         for node in ismaster.get('hosts', [])]
                 nodes.extend([partition_node(node.lower())
-                              for node in self.ismaster.get('passives', [])])
+                              for node in ismaster.get('passives', [])])
                 nodes.extend([partition_node(node.lower())
-                              for node in self.ismaster.get('arbiters', [])])
+                              for node in ismaster.get('arbiters', [])])
                 self.nodes = set(nodes)
             else:
-                self.ismaster = ismaster
                 self.nodes = set([(host, port)])
-            self.w = len(self.ismaster.get("hosts", [])) or 1
+            self.w = len(ismaster.get("hosts", [])) or 1
             self.version = Version.from_client(self.client)
 
             if 'enableTestCommands=1' in self.cmd_line['argv']:
@@ -300,6 +309,17 @@ class ClientContext(object):
 
             self.is_mongos = (self.ismaster.get('msg') == 'isdbgrid')
             self.has_ipv6 = self._server_started_with_ipv6()
+            if self.is_mongos:
+                # Check for another mongos on the next port.
+                address = self.client.address
+                next_address = address[0], address[1] + 1
+                self.mongoses.append(address)
+                mongos_client = self._connect(*next_address,
+                                              **self.default_client_options)
+                if mongos_client:
+                    ismaster = mongos_client.admin.command('ismaster')
+                    if ismaster.get('msg') == 'isdbgrid':
+                        self.mongoses.append(next_address)
 
     def init(self):
         with self.conn_lock:
@@ -333,6 +353,14 @@ class ClientContext(object):
             return False
         return bool(len(self.client.secondaries))
 
+    @property
+    def storage_engine(self):
+        try:
+            return self.server_status.get("storageEngine", {}).get("name")
+        except AttributeError:
+            # Raised if self.server_status is None.
+            return None
+
     def _check_user_provided(self):
         """Return True if db_user/db_password is already an admin user."""
         client = pymongo.MongoClient(
@@ -433,14 +461,7 @@ class ClientContext(object):
         def is_not_mmap():
             if self.is_mongos:
                 return True
-            try:
-                storage_engine = self.server_status.get(
-                    'storageEngine').get('name')
-            except AttributeError:
-                # Raised if the storageEngine key does not exist or if
-                # self.server_status is None.
-                return False
-            return storage_engine != 'mmapv1'
+            return self.storage_engine != 'mmapv1'
 
         return self._require(
             is_not_mmap, "Storage engine must not be MMAPv1", func=func)
@@ -512,6 +533,13 @@ class ClientContext(object):
                              "Must be connected to a mongos",
                              func=func)
 
+    def require_multiple_mongoses(self, func):
+        """Run a test only if the client is connected to a sharded cluster
+        that has 2 mongos nodes."""
+        return self._require(lambda: len(self.mongoses) > 1,
+                             "Must have multiple mongoses available",
+                             func=func)
+
     def require_standalone(self, func):
         """Run a test only if the client is connected to a standalone."""
         return self._require(lambda: not (self.is_mongos or self.is_rs),
@@ -532,20 +560,23 @@ class ClientContext(object):
                              "Auth with sharding requires MongoDB >= 2.0.0",
                              func=func)
 
+    def is_topology_type(self, topologies):
+        if 'single' in topologies and not (self.is_mongos or self.is_rs):
+            return True
+        if 'replicaset' in topologies and self.is_rs:
+            return True
+        if 'sharded' in topologies and self.is_mongos:
+            return True
+        return False
+
     def require_cluster_type(self, topologies=[]):
         """Run a test only if the client is connected to a cluster that
         conforms to one of the specified topologies. Acceptable topologies
         are 'single', 'replicaset', and 'sharded'."""
         def _is_valid_topology():
-            if 'single' in topologies and not (self.is_mongos or self.is_rs):
-                return True
-            if 'replicaset' in topologies and self.is_rs:
-                return True
-            if 'sharded' in topologies and self.is_mongos:
-                return True
-            return False
+            return self.is_topology_type(topologies)
         return self._require(
-            _is_valid_topology, 
+            _is_valid_topology,
             "Cluster type not in %s" % (topologies))
 
     def require_test_commands(self, func):
@@ -591,20 +622,45 @@ class ClientContext(object):
                              "Sessions not supported",
                              func=func)
 
+    def supports_transactions(self):
+        if self.storage_engine == 'mmapv1':
+            return False
+
+        if self.version.at_least(4, 1, 8):
+            return self.is_mongos or self.is_rs
+
+        if self.version.at_least(4, 0):
+            return self.is_rs
+
+        return False
+
     def require_transactions(self, func):
         """Run a test only if the deployment might support transactions.
 
         *Might* because this does not test the storage engine or FCV.
         """
-        new_func = self.require_version_min(4, 0, 0, -1)(func)
-        return self.require_replica_set(new_func)
+        return self._require(self.supports_transactions,
+                             "Transactions are not supported",
+                             func=func)
+
+    def mongos_seeds(self):
+        return ','.join('%s:%s' % address for address in self.mongoses)
 
     @property
     def supports_reindex(self):
         """Does the connected server support reindex?"""
-        return not (
-            self.version.at_least(4, 1, 0) and
-            self.is_mongos)
+        return not (self.version.at_least(4, 1, 0) and self.is_mongos)
+
+    @property
+    def supports_getpreverror(self):
+        """Does the connected server support getpreverror?"""
+        return not (self.version.at_least(4, 1, 0) or self.is_mongos)
+
+    @property
+    def requires_hint_with_min_max_queries(self):
+        """Does the server require a hint with min/max queries."""
+        # Changed in SERVER-39567.
+        return self.version.at_least(4, 1, 10)
 
 
 # Reusable client context
@@ -661,7 +717,7 @@ class IntegrationTest(PyMongoTestCase):
 # Use assertRaisesRegex if available, otherwise use Python 2.7's
 # deprecated assertRaisesRegexp, with a 'p'.
 if not hasattr(unittest.TestCase, 'assertRaisesRegex'):
-    IntegrationTest.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
+    unittest.TestCase.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
 
 
 class MockClientTest(unittest.TestCase):
@@ -723,3 +779,11 @@ def test_cases(suite):
             # unittest.TestSuite
             for case in test_cases(suite_or_case):
                 yield case
+
+
+# Helper method to workaround https://bugs.python.org/issue21724
+def clear_warning_registry():
+    """Clear the __warningregistry__ for all modules."""
+    for name, module in list(sys.modules.items()):
+        if hasattr(module, "__warningregistry__"):
+            setattr(module, "__warningregistry__", {})
