@@ -15,6 +15,7 @@
 """Test the mongo_client module."""
 
 import contextlib
+import copy
 import datetime
 import gc
 import os
@@ -28,14 +29,16 @@ import warnings
 sys.path[0:0] = [""]
 
 from bson import BSON
-from bson.codec_options import CodecOptions
+from bson.codec_options import CodecOptions, TypeEncoder, TypeRegistry
 from bson.py3compat import thread
 from bson.son import SON
 from bson.tz_util import utc
+import pymongo
 from pymongo import auth, message
-from pymongo.common import _UUID_REPRESENTATIONS
+from pymongo.common import CONNECT_TIMEOUT, _UUID_REPRESENTATIONS
 from pymongo.command_cursor import CommandCursor
-from pymongo.cursor import CursorType
+from pymongo.compression_support import _HAVE_SNAPPY, _HAVE_ZSTD
+from pymongo.cursor import Cursor, CursorType
 from pymongo.database import Database
 from pymongo.errors import (AutoReconnect,
                             ConfigurationError,
@@ -48,11 +51,13 @@ from pymongo.errors import (AutoReconnect,
 from pymongo.monitoring import (ServerHeartbeatListener,
                                 ServerHeartbeatStartedEvent)
 from pymongo.mongo_client import MongoClient
+from pymongo.driver_info import DriverInfo
 from pymongo.pool import SocketInfo, _METADATA
 from pymongo.read_preferences import ReadPreference
 from pymongo.server_selectors import (any_server_selector,
                                       writable_server_selector)
 from pymongo.server_type import SERVER_TYPE
+from pymongo.srv_resolver import _HAVE_DNSPYTHON
 from pymongo.write_concern import WriteConcern
 from test import (client_context,
                   client_knobs,
@@ -67,6 +72,7 @@ from test.pymongo_mocks import MockClient
 from test.utils import (assertRaisesExactly,
                         connected,
                         delay,
+                        FunctionCallRecorder,
                         get_pool,
                         gevent_monkey_patched,
                         ignore_deprecations,
@@ -176,6 +182,42 @@ class ClientUnitTest(unittest.TestCase):
 
         self.assertRaises(TypeError, iterate)
 
+    def test_get_default_database(self):
+        c = rs_or_single_client("mongodb://%s:%d/foo" % (client_context.host,
+                                                         client_context.port),
+                                connect=False)
+        self.assertEqual(Database(c, 'foo'), c.get_default_database())
+        # Test that default doesn't override the URI value.
+        self.assertEqual(Database(c, 'foo'), c.get_default_database('bar'))
+
+        codec_options = CodecOptions(tz_aware=True)
+        write_concern = WriteConcern(w=2, j=True)
+        db = c.get_default_database(
+            None, codec_options, ReadPreference.SECONDARY, write_concern)
+        self.assertEqual('foo', db.name)
+        self.assertEqual(codec_options, db.codec_options)
+        self.assertEqual(ReadPreference.SECONDARY, db.read_preference)
+        self.assertEqual(write_concern, db.write_concern)
+
+        c = rs_or_single_client("mongodb://%s:%d/" % (client_context.host,
+                                                      client_context.port),
+                                connect=False)
+        self.assertEqual(Database(c, 'foo'), c.get_default_database('foo'))
+
+    def test_get_default_database_error(self):
+        # URI with no database.
+        c = rs_or_single_client("mongodb://%s:%d/" % (client_context.host,
+                                                      client_context.port),
+                                connect=False)
+        self.assertRaises(ConfigurationError, c.get_default_database)
+
+    def test_get_default_database_with_authsource(self):
+        # Ensure we distinguish database name from authSource.
+        uri = "mongodb://%s:%d/foo?authSource=src" % (
+            client_context.host, client_context.port)
+        c = rs_or_single_client(uri, connect=False)
+        self.assertEqual(Database(c, 'foo'), c.get_default_database())
+
     def test_get_database_default(self):
         c = rs_or_single_client("mongodb://%s:%d/foo" % (client_context.host,
                                                          client_context.port),
@@ -212,7 +254,7 @@ class ClientUnitTest(unittest.TestCase):
         self.assertEqual(c.read_preference, ReadPreference.NEAREST)
 
     def test_metadata(self):
-        metadata = _METADATA.copy()
+        metadata = copy.deepcopy(_METADATA)
         metadata['application'] = {'name': 'foobar'}
         client = MongoClient(
             "mongodb://foo:27017/?appname=foobar&connect=false")
@@ -224,16 +266,49 @@ class ClientUnitTest(unittest.TestCase):
         # No error
         MongoClient(appname='x' * 128)
         self.assertRaises(ValueError, MongoClient, appname='x' * 129)
+        # Bad "driver" options.
+        self.assertRaises(TypeError, DriverInfo, 'Foo', 1, 'a')
+        self.assertRaises(TypeError, MongoClient, driver=1)
+        self.assertRaises(TypeError, MongoClient, driver='abc')
+        self.assertRaises(TypeError, MongoClient, driver=('Foo', '1', 'a'))
+        # Test appending to driver info.
+        metadata['driver']['name'] = 'PyMongo|FooDriver'
+        metadata['driver']['version'] = '%s|1.2.3' % (
+            _METADATA['driver']['version'],)
+        client = MongoClient('foo', 27017, appname='foobar',
+                             driver=DriverInfo('FooDriver', '1.2.3', None), connect=False)
+        options = client._MongoClient__options
+        self.assertEqual(options.pool_options.metadata, metadata)
+        metadata['platform'] = '%s|FooPlatform' % (
+            _METADATA['platform'],)
+        client = MongoClient('foo', 27017, appname='foobar',
+                             driver=DriverInfo('FooDriver', '1.2.3', 'FooPlatform'), connect=False)
+        options = client._MongoClient__options
+        self.assertEqual(options.pool_options.metadata, metadata)
 
     def test_kwargs_codec_options(self):
+        class MyFloatType(object):
+            def __init__(self, x):
+                self.__x = x
+            @property
+            def x(self):
+                return self.__x
+
+        class MyFloatAsIntEncoder(TypeEncoder):
+            python_type = MyFloatType
+            def transform_python(self, value):
+                return int(value)
+
         # Ensure codec options are passed in correctly
         document_class = SON
+        type_registry = TypeRegistry([MyFloatAsIntEncoder()])
         tz_aware = True
         uuid_representation_label = 'javaLegacy'
         unicode_decode_error_handler = 'ignore'
         tzinfo = utc
         c = MongoClient(
             document_class=document_class,
+            type_registry=type_registry,
             tz_aware=tz_aware,
             uuidrepresentation=uuid_representation_label,
             unicode_decode_error_handler=unicode_decode_error_handler,
@@ -242,6 +317,7 @@ class ClientUnitTest(unittest.TestCase):
         )
 
         self.assertEqual(c.codec_options.document_class, document_class)
+        self.assertEqual(c.codec_options.type_registry, type_registry)
         self.assertEqual(c.codec_options.tz_aware, tz_aware)
         self.assertEqual(
             c.codec_options.uuid_representation,
@@ -271,6 +347,78 @@ class ClientUnitTest(unittest.TestCase):
             c.codec_options.unicode_decode_error_handler,
             unicode_decode_error_handler)
 
+    def test_uri_option_precedence(self):
+        # Ensure kwarg options override connection string options.
+        uri = ("mongodb://localhost/?ssl=true&replicaSet=name"
+               "&readPreference=primary")
+        c = MongoClient(uri, ssl=False, replicaSet="newname",
+                        readPreference="secondaryPreferred")
+        clopts = c._MongoClient__options
+        opts = clopts._options
+
+        self.assertEqual(opts['ssl'], False)
+        self.assertEqual(clopts.replica_set_name, "newname")
+        self.assertEqual(
+            clopts.read_preference, ReadPreference.SECONDARY_PREFERRED)
+
+    @unittest.skipUnless(
+        _HAVE_DNSPYTHON, "DNS-related tests need dnspython to be installed")
+    def test_connection_timeout_ms_propagates_to_DNS_resolver(self):
+        # Patch the resolver.
+        from pymongo.srv_resolver import resolver
+        patched_resolver = FunctionCallRecorder(resolver.query)
+        pymongo.srv_resolver.resolver.query = patched_resolver
+        def reset_resolver():
+            pymongo.srv_resolver.resolver.query = resolver.query
+        self.addCleanup(reset_resolver)
+
+        # Setup.
+        base_uri = "mongodb+srv://test5.test.build.10gen.cc"
+        connectTimeoutMS = 5000
+        expected_kw_value = 5.0
+        uri_with_timeout = base_uri + "/?connectTimeoutMS=6000"
+        expected_uri_value = 6.0
+
+        def test_scenario(args, kwargs, expected_value):
+            patched_resolver.reset()
+            MongoClient(*args, **kwargs)
+            for _, kw in patched_resolver.call_list():
+                self.assertAlmostEqual(kw['lifetime'], expected_value)
+
+        # No timeout specified.
+        test_scenario((base_uri,), {}, CONNECT_TIMEOUT)
+
+        # Timeout only specified in connection string.
+        test_scenario((uri_with_timeout,), {}, expected_uri_value)
+
+        # Timeout only specified in keyword arguments.
+        kwarg = {'connectTimeoutMS': connectTimeoutMS}
+        test_scenario((base_uri,), kwarg, expected_kw_value)
+
+        # Timeout specified in both kwargs and connection string.
+        test_scenario((uri_with_timeout,), kwarg, expected_kw_value)
+
+    def test_uri_security_options(self):
+        # Ensure that we don't silently override security-related options.
+        with self.assertRaises(InvalidURI):
+            MongoClient('mongodb://localhost/?ssl=true', tls=False,
+                        connect=False)
+
+        # Matching SSL and TLS options should not cause errors.
+        c = MongoClient('mongodb://localhost/?ssl=false', tls=False,
+                        connect=False)
+        self.assertEqual(c._MongoClient__options._options['ssl'], False)
+
+        # Conflicting tlsInsecure options should raise an error.
+        with self.assertRaises(InvalidURI):
+            MongoClient('mongodb://localhost/?tlsInsecure=true',
+                        connect=False, tlsAllowInvalidHostnames=True)
+
+        # Conflicting legacy tlsInsecure options should also raise an error.
+        with self.assertRaises(InvalidURI):
+            MongoClient('mongodb://localhost/?tlsInsecure=true',
+                        connect=False, ssl_cert_reqs=True)
+
 
 class TestClient(IntegrationTest):
 
@@ -345,7 +493,7 @@ class TestClient(IntegrationTest):
 
             # Assert that if a socket is closed, a new one takes its place
             with server._pool.get_socket({}) as sock_info:
-                sock_info.close()
+                sock_info.close_socket(None)
             wait_until(lambda: 10 == len(server._pool.sockets),
                        "a closed socket gets replaced from the pool")
             self.assertFalse(sock_info in server._pool.sockets)
@@ -382,7 +530,7 @@ class TestClient(IntegrationTest):
         port are not overloaded.
         """
         host, port = client_context.host, client_context.port
-        kwargs = client_context.ssl_client_options.copy()
+        kwargs = client_context.default_client_options.copy()
         if client_context.auth_enabled:
             kwargs['username'] = db_user
             kwargs['password'] = db_pwd
@@ -501,9 +649,6 @@ class TestClient(IntegrationTest):
         self.assertEqual(eval(the_repr), client)
 
     def test_getters(self):
-        if (client_context.version >= (3, 7, 2)
-                and client_context.auth_enabled and client_context.is_rs):
-            raise SkipTest("Disabled due to SERVER-32845")
         wait_until(lambda: client_context.nodes == self.client.nodes,
                    "find all nodes")
 
@@ -555,7 +700,7 @@ class TestClient(IntegrationTest):
 
         self.client.pymongo_test.test.insert_one({"dummy": u"object"})
         self.client.pymongo_test2.test.insert_one({"dummy": u"object"})
-        dbs = self.client.database_names()
+        dbs = self.client.list_database_names()
         self.assertIn("pymongo_test", dbs)
         self.assertIn("pymongo_test2", dbs)
         self.client.drop_database("pymongo_test")
@@ -569,7 +714,7 @@ class TestClient(IntegrationTest):
 
         raise SkipTest("This test often fails due to SERVER-2329")
 
-        dbs = self.client.database_names()
+        dbs = self.client.list_database_names()
         self.assertNotIn("pymongo_test", dbs)
         self.assertNotIn("pymongo_test2", dbs)
 
@@ -579,12 +724,12 @@ class TestClient(IntegrationTest):
         self.client.close()
         self.client.close()
 
-        coll.count()
+        coll.count_documents({})
 
         self.client.close()
         self.client.close()
 
-        coll.count()
+        coll.count_documents({})
 
     def test_close_kills_cursors(self):
         if sys.platform.startswith('java'):
@@ -621,6 +766,30 @@ class TestClient(IntegrationTest):
         self.client._process_periodic_tasks()
         self.assertFalse(self.client._topology._opened)
 
+    def test_close_stops_kill_cursors_thread(self):
+        client = rs_client()
+        client.test.test.find_one()
+        self.assertFalse(client._kill_cursors_executor._stopped)
+
+        # Closing the client should stop the thread.
+        client.close()
+        self.assertTrue(client._kill_cursors_executor._stopped)
+
+        # Reusing the closed client should restart the thread.
+        client.admin.command('isMaster')
+        self.assertFalse(client._kill_cursors_executor._stopped)
+
+        # Again, closing the client should stop the thread.
+        client.close()
+        self.assertTrue(client._kill_cursors_executor._stopped)
+
+    def test_close_does_not_open_servers(self):
+        client = rs_client(connect=False)
+        topology = client._topology
+        self.assertEqual(topology._servers, {})
+        client.close()
+        self.assertEqual(topology._servers, {})
+
     def test_bad_uri(self):
         with self.assertRaises(InvalidURI):
             MongoClient("http://localhost")
@@ -743,7 +912,7 @@ class TestClient(IntegrationTest):
         # Confirm we can do operations via the socket.
         client = rs_or_single_client(uri)
         client.pymongo_test.test.insert_one({"dummy": "object"})
-        dbs = client.database_names()
+        dbs = client.list_database_names()
         self.assertTrue("pymongo_test" in dbs)
 
         self.assertTrue(mongodb_socket in repr(client))
@@ -863,8 +1032,8 @@ class TestClient(IntegrationTest):
             with warnings.catch_warnings(record=True) as ctx:
                 warnings.simplefilter("always")
                 client = rs_or_single_client(socketKeepAlive=socketKeepAlive)
-                self.assertIn("The socketKeepAlive option is deprecated",
-                              str(ctx[0]))
+                self.assertTrue(any("The socketKeepAlive option is deprecated"
+                                    in str(k) for k in ctx))
                 pool = get_pool(client)
                 self.assertEqual(socketKeepAlive,
                                  pool.opts.socket_keepalive)
@@ -892,9 +1061,6 @@ class TestClient(IntegrationTest):
     @client_context.require_ipv6
     def test_ipv6(self):
         if client_context.ssl:
-            # http://bugs.python.org/issue13034
-            if sys.version_info[:2] == (2, 6):
-                raise SkipTest("Python 2.6 can't parse SANs")
             if not HAVE_IPADDRESS:
                 raise SkipTest("Need the ipaddress module to test with SSL")
 
@@ -911,7 +1077,7 @@ class TestClient(IntegrationTest):
         client.pymongo_test.test.insert_one({"dummy": u"object"})
         client.pymongo_test_bernie.test.insert_one({"dummy": u"object"})
 
-        dbs = client.database_names()
+        dbs = client.list_database_names()
         self.assertTrue("pymongo_test" in dbs)
         self.assertTrue("pymongo_test_bernie" in dbs)
 
@@ -938,6 +1104,14 @@ class TestClient(IntegrationTest):
             time.sleep(1)
         self.assertFalse(locked)
 
+    def test_is_locked_does_not_raise_warning(self):
+        client = rs_or_single_client()
+        with warnings.catch_warnings(record=True) as ctx:
+            warnings.simplefilter("always")
+            _ = client.is_locked
+            self.assertFalse(
+                any(issubclass(w.category, DeprecationWarning) for w in ctx))
+
     def test_contextlib(self):
         client = rs_or_single_client()
         client.pymongo_test.drop_collection("test")
@@ -1051,20 +1225,33 @@ class TestClient(IntegrationTest):
 
         # Use a separate collection to avoid races where we're still
         # completing an operation on a collection while the next test begins.
+        client_context.client.drop_database('test_lazy_connect_w0')
+        self.addCleanup(
+            client_context.client.drop_database, 'test_lazy_connect_w0')
+
         client = rs_or_single_client(connect=False, w=0)
         client.test_lazy_connect_w0.test.insert_one({})
+        wait_until(
+            lambda: client.test_lazy_connect_w0.test.count_documents({}) == 1,
+            "find one document")
 
-        client = rs_or_single_client(connect=False)
+        client = rs_or_single_client(connect=False, w=0)
         client.test_lazy_connect_w0.test.update_one({}, {'$set': {'x': 1}})
+        wait_until(
+            lambda: client.test_lazy_connect_w0.test.find_one().get('x') == 1,
+            "update one document")
 
-        client = rs_or_single_client(connect=False)
+        client = rs_or_single_client(connect=False, w=0)
         client.test_lazy_connect_w0.test.delete_one({})
+        wait_until(
+            lambda: client.test_lazy_connect_w0.test.count_documents({}) == 0,
+            "delete one document")
 
     @client_context.require_no_mongos
     def test_exhaust_network_error(self):
         # When doing an exhaust query, the socket stays checked out on success
         # but must be checked in on error to avoid semaphore leaks.
-        client = rs_or_single_client(maxPoolSize=1)
+        client = rs_or_single_client(maxPoolSize=1, retryReads=False)
         collection = client.pymongo_test.test
         pool = get_pool(client)
         pool._check_interval_seconds = None  # Never check.
@@ -1091,11 +1278,12 @@ class TestClient(IntegrationTest):
 
         # Get a client with one socket so we detect if it's leaked.
         c = connected(rs_or_single_client(maxPoolSize=1,
-                                          waitQueueTimeoutMS=1))
+                                          waitQueueTimeoutMS=1,
+                                          retryReads=False))
 
         # Simulate an authenticate() call on a different socket.
         credentials = auth._build_credentials_tuple(
-            'DEFAULT', 'admin', db_user, db_pwd, {})
+            'DEFAULT', 'admin', db_user, db_pwd, {}, None)
 
         c._cache_credentials('test', credentials, connect=False)
 
@@ -1123,14 +1311,17 @@ class TestClient(IntegrationTest):
     def test_stale_getmore(self):
         # A cursor is created, but its member goes down and is removed from
         # the topology before the getMore message is sent. Test that
-        # MongoClient._send_message_with_response handles the error.
+        # MongoClient._run_operation_with_response handles the error.
         with self.assertRaises(AutoReconnect):
             client = rs_client(connect=False,
                                serverSelectionTimeoutMS=100)
-            client._send_message_with_response(
+            client._run_operation_with_response(
                 operation=message._GetMore('pymongo_test', 'collection',
                                            101, 1234, client.codec_options,
-                                           None, client),
+                                           ReadPreference.PRIMARY,
+                                           None, client, None, None),
+                unpack_res=Cursor(
+                    client.pymongo_test.collection)._unpack_response,
                 address=('not-a-member', 27017))
 
     def test_heartbeat_frequency_ms(self):
@@ -1179,6 +1370,91 @@ class TestClient(IntegrationTest):
 
         self.assertIn('heartbeatFrequencyMS', str(context.exception))
 
+    def test_compression(self):
+        def compression_settings(client):
+            pool_options = client._MongoClient__options.pool_options
+            return pool_options.compression_settings
+
+        uri = "mongodb://localhost:27017/?compressors=zlib"
+        client = MongoClient(uri, connect=False)
+        opts = compression_settings(client)
+        self.assertEqual(opts.compressors, ['zlib'])
+        uri = "mongodb://localhost:27017/?compressors=zlib&zlibCompressionLevel=4"
+        client = MongoClient(uri, connect=False)
+        opts = compression_settings(client)
+        self.assertEqual(opts.compressors, ['zlib'])
+        self.assertEqual(opts.zlib_compression_level, 4)
+        uri = "mongodb://localhost:27017/?compressors=zlib&zlibCompressionLevel=-1"
+        client = MongoClient(uri, connect=False)
+        opts = compression_settings(client)
+        self.assertEqual(opts.compressors, ['zlib'])
+        self.assertEqual(opts.zlib_compression_level, -1)
+        uri = "mongodb://localhost:27017"
+        client = MongoClient(uri, connect=False)
+        opts = compression_settings(client)
+        self.assertEqual(opts.compressors, [])
+        self.assertEqual(opts.zlib_compression_level, -1)
+        uri = "mongodb://localhost:27017/?compressors=foobar"
+        client = MongoClient(uri, connect=False)
+        opts = compression_settings(client)
+        self.assertEqual(opts.compressors, [])
+        self.assertEqual(opts.zlib_compression_level, -1)
+        uri = "mongodb://localhost:27017/?compressors=foobar,zlib"
+        client = MongoClient(uri, connect=False)
+        opts = compression_settings(client)
+        self.assertEqual(opts.compressors, ['zlib'])
+        self.assertEqual(opts.zlib_compression_level, -1)
+
+        # According to the connection string spec, unsupported values
+        # just raise a warning and are ignored.
+        uri = "mongodb://localhost:27017/?compressors=zlib&zlibCompressionLevel=10"
+        client = MongoClient(uri, connect=False)
+        opts = compression_settings(client)
+        self.assertEqual(opts.compressors, ['zlib'])
+        self.assertEqual(opts.zlib_compression_level, -1)
+        uri = "mongodb://localhost:27017/?compressors=zlib&zlibCompressionLevel=-2"
+        client = MongoClient(uri, connect=False)
+        opts = compression_settings(client)
+        self.assertEqual(opts.compressors, ['zlib'])
+        self.assertEqual(opts.zlib_compression_level, -1)
+
+        if not _HAVE_SNAPPY:
+            uri = "mongodb://localhost:27017/?compressors=snappy"
+            client = MongoClient(uri, connect=False)
+            opts = compression_settings(client)
+            self.assertEqual(opts.compressors, [])
+        else:
+            uri = "mongodb://localhost:27017/?compressors=snappy"
+            client = MongoClient(uri, connect=False)
+            opts = compression_settings(client)
+            self.assertEqual(opts.compressors, ['snappy'])
+            uri = "mongodb://localhost:27017/?compressors=snappy,zlib"
+            client = MongoClient(uri, connect=False)
+            opts = compression_settings(client)
+            self.assertEqual(opts.compressors, ['snappy', 'zlib'])
+
+        if not _HAVE_ZSTD:
+            uri = "mongodb://localhost:27017/?compressors=zstd"
+            client = MongoClient(uri, connect=False)
+            opts = compression_settings(client)
+            self.assertEqual(opts.compressors, [])
+        else:
+            uri = "mongodb://localhost:27017/?compressors=zstd"
+            client = MongoClient(uri, connect=False)
+            opts = compression_settings(client)
+            self.assertEqual(opts.compressors, ['zstd'])
+            uri = "mongodb://localhost:27017/?compressors=zstd,zlib"
+            client = MongoClient(uri, connect=False)
+            opts = compression_settings(client)
+            self.assertEqual(opts.compressors, ['zstd', 'zlib'])
+
+        options = client_context.default_client_options
+        if "compressors" in options and "zlib" in options["compressors"]:
+            for level in range(-1, 10):
+                client = single_client(zlibcompressionlevel=level)
+                # No error
+                client.pymongo_test.test.find_one()
+
 
 class TestExhaustCursor(IntegrationTest):
     """Test that clients properly handle errors from exhaust cursors."""
@@ -1245,13 +1521,14 @@ class TestExhaustCursor(IntegrationTest):
         sock_info.receive_message = saved
 
         # The socket is returned the pool and it still works.
-        self.assertEqual(200, collection.count())
+        self.assertEqual(200, collection.count_documents({}))
         self.assertIn(sock_info, pool.sockets)
 
     def test_exhaust_query_network_error(self):
         # When doing an exhaust query, the socket stays checked out on success
         # but must be checked in on error to avoid semaphore leaks.
-        client = connected(rs_or_single_client(maxPoolSize=1))
+        client = connected(rs_or_single_client(maxPoolSize=1,
+                                               retryReads=False))
         collection = client.pymongo_test.test
         pool = get_pool(client)
         pool._check_interval_seconds = None  # Never check.
@@ -1310,7 +1587,7 @@ class TestClientLazyConnect(IntegrationTest):
             collection.insert_one({})
 
         def test(collection):
-            self.assertEqual(NTHREADS, collection.count())
+            self.assertEqual(NTHREADS, collection.count_documents({}))
 
         lazy_client_trial(reset, insert_one, test, self._get_client)
 
@@ -1337,7 +1614,7 @@ class TestClientLazyConnect(IntegrationTest):
             collection.delete_one({'i': i})
 
         def test(collection):
-            self.assertEqual(0, collection.count())
+            self.assertEqual(0, collection.count_documents({}))
 
         lazy_client_trial(reset, delete_one, test, self._get_client)
 
@@ -1408,7 +1685,8 @@ class TestMongoClientFailover(MockClientTest):
             members=['a:1', 'b:2', 'c:3'],
             mongoses=[],
             host='b:2',  # Pass a secondary.
-            replicaSet='rs')
+            replicaSet='rs',
+            retryReads=False)
 
         wait_until(lambda: len(c.nodes) == 3, 'connect')
 
@@ -1436,7 +1714,8 @@ class TestMongoClientFailover(MockClientTest):
                 mongoses=[],
                 host='a:1',
                 replicaSet='rs',
-                connect=False)
+                connect=False,
+                retryReads=False)
 
             # Set host-specific information so we can test whether it is reset.
             c.set_wire_version_range('a:1', 2, 6)
