@@ -53,10 +53,19 @@ class MockPool(object):
         self.pool_id = 0
         self._lock = threading.Lock()
 
-    def reset(self):
+    def _reset(self):
         with self._lock:
             self.pool_id += 1
 
+    def reset(self):
+        self._reset()
+
+    def close(self):
+        self._reset()
+
+    def update_is_writable(self, is_writable):
+        pass
+
 
 class MockMonitor(object):
     def __init__(self, server_description, topology, pool, topology_settings):
