@@ -663,6 +663,7 @@ class TestSampleShellCommands(unittest.TestCase):
 
     @client_context.require_version_min(3, 5, 11)
     @client_context.require_replica_set
+    @client_context.require_no_mmap
     def test_change_streams(self):
         db = client_context.client.pymongo_test
         done = False
@@ -688,7 +689,7 @@ class TestSampleShellCommands(unittest.TestCase):
             # End Changestream Example 2
 
             # Start Changestream Example 3
-            resume_token = document.get("_id")
+            resume_token = cursor.resume_token
             cursor = db.inventory.watch(resume_after=resume_token)
             document = next(cursor)
             # End Changestream Example 3
@@ -1010,7 +1011,8 @@ class TestTransactionExamples(IntegrationTest):
 
             with session.start_transaction(
                     read_concern=ReadConcern("snapshot"),
-                    write_concern=WriteConcern(w="majority")):
+                    write_concern=WriteConcern(w="majority"),
+                    read_preference=ReadPreference.PRIMARY):
                 employees_coll.update_one(
                     {"employee": 3}, {"$set": {"status": "Inactive"}},
                     session=session)
@@ -1116,6 +1118,7 @@ class TestTransactionExamples(IntegrationTest):
 class TestCausalConsistencyExamples(IntegrationTest):
     @client_context.require_version_min(3, 6, 0)
     @client_context.require_secondaries_count(1)
+    @client_context.require_no_mmap
     def test_causal_consistency(self):
         # Causal consistency examples
         client = self.client
